package test;

/*
 *  Copyright (C) 2020 Juergen Platzer
 *
 *  This file is part of JInfoViz.
 *
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */


import tools.CovarianceCalculator;
import tools.SymmetricMatrix;
import java.util.Vector;

public class CovarianceCalculatorTest extends AbstractTest {



  public CovarianceCalculatorTest() {
    initTest();
  }


  protected String getTestName() {
    return "Covariance Calculation Test";
  }



  protected void performActualTest() {
    testCase1();
    testCase2();
    testIris();
    testFirstTenIris();
    testLastTenIris();

  }


  ////////////////////////////////////////
  // PRIVATE METHODS                    //
  ////////////////////////////////////////

  /*
      x = c(1:10)
    > y = c(10:1)
    > var(x)
    [1] 9.166667
    > var(y)
    [1] 9.166667
    > cov(x,y)
    [1] -9.166667


  */

  private void testCase1() {
    SymmetricMatrix expectedMatrix = new SymmetricMatrix(2);
    expectedMatrix.setValue(0,0,9.166667);
    expectedMatrix.setValue(0,1,-9.166667);
    expectedMatrix.setValue(1,1,9.166667);


    SymmetricMatrix covMatrix = getCalculatedCovarianceMatrix(getTestCase1Data());
    performTestComparison("Covariance matrix test 1", expectedMatrix, covMatrix, 0.000001);


  }


  private static float[][] getTestCase1Data() {
    int entryNum = 10;
    float[][] data = new float[2][entryNum];
    for (int i = 0; i < entryNum; i++) {
      data[0][i] = (float)(i+1); //1 - 10;
      data[1][i] = (float)(entryNum-i); //10 - 1;
    }
    return data;
  }


  /**

    > x = c(1:10)
    > y = 3*x + 2
    > B <- matrix(c(x,y), nrow=10, byrow=FALSE)
    > B
        [,1] [,2]
    [1,]    1    5
    [2,]    2    8
    [3,]    3   11
    [4,]    4   14
    [5,]    5   17
    [6,]    6   20
    [7,]    7   23
    [8,]    8   26
    [9,]    9   29
    [10,]   10   32
    > cov(B)
            [,1] [,2]
    [1,]  9.166667 27.5
    [2,] 27.500000 82.5
    > det(cov(B))
    [1] 0
  **/

  private void testCase2() {
    SymmetricMatrix expectedMatrix = new SymmetricMatrix(2);
    expectedMatrix.setValue(0,0,9.166667);
    expectedMatrix.setValue(0,1,27.5);
    expectedMatrix.setValue(1,1,82.5);


    SymmetricMatrix covMatrix = getCalculatedCovarianceMatrix(getTestCase2Data());
    performTestComparison("Covariance matrix test 2", expectedMatrix, covMatrix, 0.000001);


  }



  private static float[][] getTestCase2Data() {
    int entryNum = 10;
    float[][] data = new float[2][entryNum];
    for (int i = 0; i < entryNum; i++) {
      data[0][i] = (float)(i+1); //1 - 10;
      data[1][i] = (float)(3*(i+1)+2); //10 - 1;
    }
    return data;
  }


  /**


  cov(iris[1:4])
                 Sepal.Length Sepal.Width Petal.Length Petal.Width
    Sepal.Length    0.6856935  -0.0424340    1.2743154   0.5162707
    Sepal.Width    -0.0424340   0.1899794   -0.3296564  -0.1216394
    Petal.Length    1.2743154  -0.3296564    3.1162779   1.2956094
    Petal.Width     0.5162707  -0.1216394    1.2956094   0.5810063
    > C = cov(iris[1:4])

  */



  private void testIris() {
    SymmetricMatrix expectedMatrix = new SymmetricMatrix(4);
    expectedMatrix.setValue(0,0,0.6856935);
    expectedMatrix.setValue(0,1,-0.0424340);
    expectedMatrix.setValue(0,2,1.2743154);
    expectedMatrix.setValue(0,3,0.5162707);
    expectedMatrix.setValue(1,1,0.1899794);
    expectedMatrix.setValue(1,2,-0.3296564);
    expectedMatrix.setValue(1,3,-0.1216394);
    expectedMatrix.setValue(2,2,3.1162779);
    expectedMatrix.setValue(2,3,1.2956094);
    expectedMatrix.setValue(3,3,0.5810063);


    SymmetricMatrix covMatrix = getCalculatedCovarianceMatrix(TestDataProvider.getIrisData());
    performTestComparison("Iris covariance matrix ", expectedMatrix, covMatrix, 0.000001);


  }


  /**
  --first 10 rows of iris dataset
    head(iris[1:4], 10)


    cov(head(iris[1:4], 10))

    cov(head(iris[1:4], 10))
               Sepal.Length Sepal.Width Petal.Length Petal.Width
    Sepal.Length  0.084888889  0.07044444  0.018888889 0.008666667
    Sepal.Width   0.070444444  0.09433333  0.017222222 0.016444444
    Petal.Length  0.018888889  0.01722222  0.011666667 0.004444444
    Petal.Width   0.008666667  0.01644444  0.004444444 0.006222222


  **/
  private void testFirstTenIris() {
    SymmetricMatrix expectedMatrix = new SymmetricMatrix(4);
    expectedMatrix.setValue(0,0,0.084888889);
    expectedMatrix.setValue(0,1,0.07044444);
    expectedMatrix.setValue(0,2,0.018888889);
    expectedMatrix.setValue(0,3,0.008666667);
    expectedMatrix.setValue(1,1,0.09433333);
    expectedMatrix.setValue(1,2,0.017222222);
    expectedMatrix.setValue(1,3,0.016444444);
    expectedMatrix.setValue(2,2,0.011666667);
    expectedMatrix.setValue(2,3,0.004444444);
    expectedMatrix.setValue(3,3,0.006222222);


    SymmetricMatrix covMatrix = getCalculatedCovarianceMatrix(TestDataProvider.getFirstTenIrisData());
    performTestComparison("First ten iris covariance matrix ", expectedMatrix, covMatrix, 0.000001);


  }


  /**
  --first 10 rows of iris dataset
    tail(iris[1:4], 10)


    cov(tail(iris[1:4], 10))
                  Sepal.Length Sepal.Width Petal.Length Petal.Width
    Sepal.Length   0.14722222  0.04277778   0.05833333  0.07277778
    Sepal.Width    0.04277778  0.07122222   0.05455556  0.04766667
    Petal.Length   0.05833333  0.05455556   0.09344444  0.05322222
    Petal.Width    0.07277778  0.04766667   0.05322222  0.06011111


  **/
  private void testLastTenIris() {
    SymmetricMatrix expectedMatrix = new SymmetricMatrix(4);
    expectedMatrix.setValue(0,0,0.14722222);
    expectedMatrix.setValue(0,1,0.04277778);
    expectedMatrix.setValue(0,2,0.05833333);
    expectedMatrix.setValue(0,3,0.07277778);
    expectedMatrix.setValue(1,1,0.07122222);
    expectedMatrix.setValue(1,2,0.05455556);
    expectedMatrix.setValue(1,3,0.04766667);
    expectedMatrix.setValue(2,2,0.09344444);
    expectedMatrix.setValue(2,3,0.05322222);
    expectedMatrix.setValue(3,3,0.06011111);


    SymmetricMatrix covMatrix = getCalculatedCovarianceMatrix(TestDataProvider.getLastTenIrisData());
    performTestComparison("Last ten iris covariance matrix ", expectedMatrix, covMatrix, 0.000001);


  }





  private SymmetricMatrix getCalculatedCovarianceMatrix(float[][] data) {
    try {
      CovarianceCalculator covCalc = new CovarianceCalculator(data);
      return covCalc.getClassicCovarianceMatrix();
    }
    catch (Exception e) {
      e.printStackTrace();
      return null;
    }

  }



}
