package test;

/*
 *  Copyright (C) 2020 Juergen Platzer
 *
 *  This file is part of JInfoViz.
 *
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */


import tools.MCDBestSolutions;
import tools.MCDSolution;
import tools.CovarianceCalculator;

import java.util.Random;

public class MCDBestSolutionsTest extends AbstractTest {

  private static final double BEST_SOLUTION_VALUE = -16.75615;

  public MCDBestSolutionsTest() {
    initTest();
  }


  protected String getTestName() {
    return "MCDBestSolution Test";
  }



  /*
  > log(det(cov(tail(iris[1:4], 10))))
[1] -12.33424
> log(det(cov(tail(head(iris[1:4], 140), 10))))
[1] -8.430171
> log(det(cov(tail(head(iris[1:4], 130), 10))))
[1] -10.71472
> log(det(cov(tail(head(iris[1:4], 120), 10))))
[1] -8.902816
> log(det(cov(tail(head(iris[1:4], 110), 10))))
[1] -10.73027
> log(det(cov(tail(head(iris[1:4], 100), 10))))
[1] -14.61582
> log(det(cov(tail(head(iris[1:4], 90), 10))))
[1] -12.10646
> log(det(cov(tail(head(iris[1:4], 80), 10))))
[1] -11.61815
> log(det(cov(tail(head(iris[1:4], 70), 10))))
[1] -10.00353
> log(det(cov(tail(head(iris[1:4], 60), 10))))
[1] -11.83967
> log(det(cov(tail(head(iris[1:4], 50), 10))))
[1] -13.26209
> log(det(cov(tail(head(iris[1:4], 40), 10))))
[1] -14.42414
> log(det(cov(tail(head(iris[1:4], 30), 10))))
[1] -13.69752
> log(det(cov(tail(head(iris[1:4], 20), 10))))
[1] -13.81206
> log(det(cov(tail(head(iris[1:4], 10), 10))))
[1] -16.75615

  Best Solution: -16.75615 -> tail(head(iris[1:4], 10), 10) -> first 10 iris datasets

*/
  protected void performActualTest() {
    fillRandomTest();
    fillRandomTest2();
    fillRandomTest3();
  }


  ////////////////////////////////////////
  // PRIVATE METHODS                    //
  ////////////////////////////////////////

  private void fillRandomTest() {
    MCDBestSolutions bestSolutions = new MCDBestSolutions(10);

    for (int i = 0; i <= 14; i++) {
      float[][] data = TestDataProvider.getIrisDataRange(i*10, i*10+9);
      CovarianceCalculator covCalc = new CovarianceCalculator(data);
      MCDSolution solution = new MCDSolution(covCalc);
      //System.out.println(solution.getSolutionValue());
      bestSolutions.updateBestSolutions(solution);
    }

    performTestComparison("Random filling 1", BEST_SOLUTION_VALUE, bestSolutions.getBestSolution().getSolutionValue(), 0.00001);
  }

  private void fillRandomTest2() {
    MCDBestSolutions bestSolutions = new MCDBestSolutions(10);

    for (int i = 14; i >= 0; i--) {
      float[][] data = TestDataProvider.getIrisDataRange(i*10, i*10+9);
      CovarianceCalculator covCalc = new CovarianceCalculator(data);
      MCDSolution solution = new MCDSolution(covCalc);
      //System.out.println(solution.getSolutionValue());
      bestSolutions.updateBestSolutions(solution);
    }

    performTestComparison("Random filling 2", BEST_SOLUTION_VALUE, bestSolutions.getBestSolution().getSolutionValue(), 0.00001);
  }

  private void fillRandomTest3() {
    MCDBestSolutions bestSolutions = new MCDBestSolutions(10);
    Random random = new Random();
    int[] randomIds = new int[11];
    randomIds[10] = 0; //best solution
    for (int i = 0; i < 10; i++) {
      randomIds[i] = random.nextInt(14);
    }

    for (int i = 0; i < randomIds.length; i++) {
      int id = randomIds[i];
      float[][] data = TestDataProvider.getIrisDataRange(id*10, id*10+9);
      CovarianceCalculator covCalc = new CovarianceCalculator(data);
      MCDSolution solution = new MCDSolution(covCalc);
      //System.out.println(solution.getSolutionValue());
      bestSolutions.updateBestSolutions(solution);
    }

    performTestComparison("Random filling 3", BEST_SOLUTION_VALUE, bestSolutions.getBestSolution().getSolutionValue(), 0.00001);
  }




}
