package test;

/*
 *  Copyright (C) 2020 Juergen Platzer
 *
 *  This file is part of JInfoViz.
 *
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */


import tools.MCDCovarianceCalculator;
import tools.SymmetricMatrix;
import java.util.Vector;

public class MCDCovarianceCalculatorTest extends AbstractTest {






  public MCDCovarianceCalculatorTest() {
    initTest();
  }


  protected String getTestName() {
    return "MCD Covariance Calculation Test";
  }



  protected void performActualTest() {
    testIris();
  }


  ////////////////////////////////////////
  // PRIVATE METHODS                    //
  ////////////////////////////////////////




  /*



  ##########################################
  MCD
  ##########################################
  > library(MASS)
  > cov.mcd(iris[1:4])
  $center
  Sepal.Length  Sepal.Width Petal.Length  Petal.Width
      6.215464     2.865979     4.851546     1.662887

  $cov
               Sepal.Length Sepal.Width Petal.Length Petal.Width
  Sepal.Length    0.3798625  0.11501074    0.3827363  0.15203823
  Sepal.Width     0.1150107  0.10435137    0.1394802  0.08132839
  Petal.Length    0.3827363  0.13948024    0.6016903  0.27297466
  Petal.Width     0.1520382  0.08132839    0.2729747  0.17964991

  $msg
  [1] "3 singular samples of size 5 out of 2500"

  $crit (the value of the criterion on log scale. For MCD this is the determinant, and for MVE it is proportional to the volume. )
  [1] -9.811189

  $best
   [1]  52  53  54  55  56  57  58  59  60  62  64  65  66  67  68  70  71  72  73
  [20]  74  75  76  77  78  79  80  81  82  83  84  85  86  87  89  90  91  92  93
  [39]  94  95  96  97  98 100 101 102 103 104 105 110 111 112 113 114 116 117 121
  [58] 122 124 125 127 128 129 133 134 137 138 139 140 141 143 144 145 147 148 149
  [77] 150

  $n.obs
  [1] 150


  */


  private void testIris() {
    SymmetricMatrix expectedMatrix = new SymmetricMatrix(4);
    expectedMatrix.setValue(0,0,0.3798625);
    expectedMatrix.setValue(0,1,0.11501074 );
    expectedMatrix.setValue(0,2,0.3827363);
    expectedMatrix.setValue(0,3,0.15203823);
    expectedMatrix.setValue(1,1,0.10435137);
    expectedMatrix.setValue(1,2,0.1394802);
    expectedMatrix.setValue(1,3,0.08132839);
    expectedMatrix.setValue(2,2,0.6016903);
    expectedMatrix.setValue(2,3,0.27297466);
    expectedMatrix.setValue(3,3,0.17964991);

    SymmetricMatrix covMatrix = null;
    double[] center = null;
    try {
      MCDCovarianceCalculator covCalc = new MCDCovarianceCalculator(0.5f, TestDataProvider.getIrisData());
      covMatrix = covCalc.getRobustCovarianceMatrix();
      center = covCalc.getRobustMeanVector();
    }
    catch (Exception e) {
      e.printStackTrace();
      return;
    }

    performTestComparison("Robust Iris covariance matrix ", expectedMatrix, covMatrix, 0.000001);

    double[] expectedCenter = new double[] {6.215464, 2.865979, 4.851546, 1.662887};
    performTestComparison("Robust Iris center ", expectedCenter, center, 0.000001);


  }





}
