package tools;

/*
 *  Copyright (C) 2018-2020 Juergen Platzer
 *
 *  This file is part of JInfoViz.
 *
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */

public class IndexAndValue implements Comparable<IndexAndValue> {

  /*** member variables ***/
  private int    _index;
  private double  _value;


  /////////////////////////
  // CONSTRUCTOR         //
  /////////////////////////
  public IndexAndValue(int index, double value) {
    _index = index;
    _value = value;
  }


  /////////////////////////////
  // COMPARABLE INTERFACE    //
  /////////////////////////////
  public int compareTo(IndexAndValue otherIndexAndValue) throws ClassCastException {
    //1. make a simple check
    if (otherIndexAndValue == null) return 1;
    //TODO: Handle NaNs



    //2. make the comparison
    double otherValue = otherIndexAndValue.getValue();
    if (Double.isNaN(otherValue) || Double.isNaN(_value)) {
      //System.out.println("Compare: " + otherValue + " vs. my " + _value);
    }


    if (_value == otherValue) return 0;
    if (_value < otherValue) return -1;
    return 1;
  }


  /////////////////////////////////
  // GET METHODS                 //
  /////////////////////////////////
  public double getValue() {
    return _value;
  }

  public int getIndex() {
    return _index;
  }

}
