package tools;

/*
 *  Copyright (C) 2020 Juergen Platzer
 *
 *  This file is part of JInfoViz.
 *
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */


public class MCDSolution {

  private CovarianceCalculator _covCalculator;

  public MCDSolution(CovarianceCalculator covCalculator) {
    _covCalculator = covCalculator;
  }

  public double getSolutionValue() {
    if (_covCalculator == null) return Double.POSITIVE_INFINITY;
    try {
      SymmetricMatrix covMatrix = _covCalculator.getClassicCovarianceMatrix();
      if (covMatrix.isNotInvertable()) return Double.POSITIVE_INFINITY;
      return Math.log(covMatrix.getDeterminant());
    }
    catch (Exception e) {
       return Double.POSITIVE_INFINITY;
    }

  }

  public CovarianceCalculator getCovarianceCalculator() {
    return _covCalculator;
  }

}
