package tools;

/*
 *  Copyright (C) 2020 Juergen Platzer
 *
 *  This file is part of JInfoViz.
 *
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.util.Vector;

public class SymmetricMatrix extends SquareMatrix{

  private MatrixInversion _inversion;

  public SymmetricMatrix(int dimension) {
    super(dimension);
    _inversion = null;
  }

  public SymmetricMatrix(double[][] values) throws Exception {

    super(values);
    _inversion = null;

    int dim = getDimension();
    for (int i = 0; i < dim; i++) {
      for (int j = i; j < dim; j++) {
        setValue(i,j, values[i][j]);
      }
    }

  }



  //////////////////////////////
  // SET METHODS              //
  //////////////////////////////
  public void setValue(int i, int j, double value) {
    super.setValue(i,j,value);
    super.setValue(j,i,value);
    _inversion = null;
  }







  //////////////////////////////
  // GET METHODS              //
  //////////////////////////////
  public SquareMatrix getInverse() throws Exception {
    initMatrixInversion();
    return _inversion.getInverse();
  }

  public double getDeterminant() throws Exception {
    initMatrixInversion();
    return _inversion.getDeterminant();
  }

  public boolean isNotInvertable() throws Exception {
    initMatrixInversion();
    return _inversion.isNotInvertable();
  }

  public Vector<EigenValueAndVector> getEigenValuesAndVectors() throws Exception {
    EigenvalueDecomposition evd = new EigenvalueDecomposition(this);
    return evd.getEigenValuesAndVectors();
  }


  //////////////////////////////////
  // PRIVATE METHODS              //
  //////////////////////////////////
  private void initMatrixInversion() throws Exception {
    //TODO: check if this is also possible for simple square matrix -> move to SquareMatrix?
    if (_inversion == null) {
      _inversion = new MatrixInversion(this);
    }
  }










}
