package tools;

/*
 *  Copyright (C) 2020 Juergen Platzer
 *
 *  This file is part of JInfoViz.
 *
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.util.Random;

public class VectorUtilities {


  public static double[] getRandomVector(int dimension) {
    double[] randomVector = new double[dimension];
    Random random = new Random();
    for (int i = 0; i < dimension; i++) {
      randomVector[i] = random.nextDouble();
    }

    return normalizeVector(randomVector);
  }


  public static double[] normalizeVector(double[] vector) {
    double dimension = vector.length;
    double norm = getNormOfVector(vector);
    //System.out.println("norm: " + norm);
    for (int i = 0; i < dimension; i++) {
      vector[i] /= norm;
    }
    return vector;
  }

  public static double getNormOfVector(double[] vector) {
    double dimension = vector.length;
    double norm = 0;
    for (int i = 0; i < dimension; i++) {
      norm += vector[i]* vector[i];
    }
    return Math.sqrt(norm);
  }

  /*
  v*vt

     x = c(1,2)
     x%*%t(x)
     [,1] [,2]
[1,]    1    2
[2,]    2    4


  1   x    1    2   =   1  2
  2                     2  4

  **/
  public static SquareMatrix getVectorMultipliedWithItsTransposed(double[] vector) throws Exception {
    int dim = vector.length;
    double[][] result = new double[dim][dim];

    for (int i = 0; i < dim; i++) {
      for (int j = 0; j < dim; j++) {
        result[i][j] = vector[i]*vector[j];
      }
    }

    return new SquareMatrix(result);


  }

}
